﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Entity;
using System.Data.Entity.Infrastructure;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Web.Http;
using System.Web.Http.ModelBinding;
using System.Web.Http.OData;
using System.Web.Http.OData.Routing;
using EJGrid.Models;
using System.Data.SqlClient;

namespace EJGrid.Controllers
{
    /*
    The WebApiConfig class may require additional changes to add a route for this controller. Merge these statements into the Register method of the WebApiConfig class as applicable. Note that OData URLs are case sensitive.

    using System.Web.Http.OData.Builder;
    using EJGrid.Models;
    ODataConventionModelBuilder builder = new ODataConventionModelBuilder();
    builder.EntitySet<Order>("Orders");
    config.Routes.MapODataRoute("odata", "odata", builder.GetEdmModel());
    */
    public class OrdersController : ODataController
    {
        NORTHWNDEntities db = new NORTHWNDEntities();

        // GET: odata/Orders
        [Queryable]
        public IQueryable<Order> GetOrders()
        {
            return db.Orders;
        }

        // GET: odata/Orders(5)
        //public SingleResult<Order> GetOrderView([FromODataUri] int key)
        //{
        //    return SingleResult.Create(db.OrderViews.Where(orderview => orderview.OrderID == key));
        //}

        // PUT odata/OrderView(5)
        public IHttpActionResult Put([FromODataUri] int key, Order orderview)
        {
            if (!ModelState.IsValid)
            {
                return BadRequest(ModelState);
            }

            if (key != orderview.OrderID)
            {
                return BadRequest();
            }

            db.Entry(orderview).State = EntityState.Modified;

            try
            {
                db.SaveChanges();
            }
            catch (DbUpdateConcurrencyException)
            {
                if (!OrderViewExists(key))
                {
                    return NotFound();
                }
                else
                {
                    throw;
                }
            }

            return Updated(orderview);
        }

        // POST odata/OrderView
        public IHttpActionResult Post(Order orderview)
        {
            if (!ModelState.IsValid)
            {
                return BadRequest(ModelState);
            }

            db.Orders.Add(orderview);

            try
            {
                db.SaveChanges();
            }
            catch (DbUpdateException)
            {
                if (OrderViewExists(orderview.OrderID))
                {
                    return Conflict();
                }
                else
                {
                    throw;
                }
            }

            return Created(orderview);
        }

        // PATCH odata/OrderView(5)
        [AcceptVerbs("PATCH", "MERGE")]
        public IHttpActionResult Patch([FromODataUri] int key, Delta<Order> patch)
        {
            if (!ModelState.IsValid)
            {
                return BadRequest(ModelState);
            }

            Order orderview = db.Orders.Find(key);
            if (orderview == null)
            {
                return NotFound();
            }

            patch.Patch(orderview);

            try
            {
                db.SaveChanges();
            }
            catch (DbUpdateConcurrencyException)
            {
                if (!OrderViewExists(key))
                {
                    return NotFound();
                }
                else
                {
                    throw;
                }
            }

            return Updated(orderview);
        }

        // DELETE odata/OrderView(5)
        public IHttpActionResult Delete([FromODataUri] int key)
        {
            Order orderview = db.Orders.Find(key);
            if (orderview == null)
            {
                return NotFound();
            }

            db.Orders.Remove(orderview);
            db.SaveChanges();

            return StatusCode(HttpStatusCode.NoContent);
        }

        protected override void Dispose(bool disposing)
        {
            if (disposing)
            {
                db.Dispose();
            }
            base.Dispose(disposing);
        }

        private bool OrderViewExists(int key)
        {
            return db.Orders.Count(e => e.OrderID == key) > 0;
        }
    }
}